/*
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights reserved.
 *
 * Author: Pops, TCSDeveloper
 */

using System;
using System.Collections;

namespace TopCoder.Rules.DependencyMap.View
{

    /// <summary>
    /// This class represents an object and it's dependants. The application can use this class to build the
    /// dependency graph that will be rendered by the DependencyDiagram. Each node can have two attributes:
    /// 1) The label (aka title) that will appear on the shape in the diagram
    /// 2) A specific shape to use for this specific node. The type should exist in the corresponding shape
    /// factory. If not found at rendering time, the common shape will appear instead.
    /// </summary>
    public class Dependency
    {

        /// <summary>
        /// This represents the title of this dependency. The title will be used by the layout manager to assign
        /// text to the eventual shape. The title will also be considered a unique identifier of the dependency
        /// (ie no other dependency should have the same title) by this component since the text of the title
        /// appears on the shape (and thus is used to uniquely identify it). This variable is set by the constructor,
        /// immutable (has been marked final to enforce immutability), and is access by the getter method. This
        /// variable has been marked readonly to enforce it's immutablility (again - reference only).
        /// </summary>
        private readonly string title;

        /// <summary>
        /// This represents a dictionary of dependency's that this dependency is dependant upon. The dictionary
        /// contains the dependant's title as the key and the Dependency as the value. The dependants are stored
        /// in a hashtable to allow quick retrievals by title (which is considered unique identifier). The
        /// dictionary may be empty if this dependency is not dependant upon anything. This variable is set at
        /// object construction time, is immutable (the reference, not the contents) and can be accessed by the
        /// add/remove/getter methods. This variable has been marked readonly to enforce it's immutablility
        /// (again - reference only).
        /// </summary>
        private readonly IDictionary dependants = new Hashtable();

        /// <summary>
        /// This represents the type of shape that should override the common shape that would have been used to
        /// render this element. The type must exist in the corresponding ShapeFactory to be rendered. This value
        /// will be -1 if there is no override. This variable is set in the constructor, is immutable once set and
        /// is access by the getter method. This variable has been marked readonly to enforce it's immutablility
        /// (again - reference only).
        /// </summary>
        private readonly int shapeOverride;

        /// <summary>
        /// This is the property representing the title variable. This is a simple getter property definition for
        /// the title.
        /// </summary>
        public string Title
        {
            get
            {
                return this.title;
            }
        }

        /// <summary>
        /// This is the property representing the shapeOverride variable. This is a simple getter ONLY property
        /// definition for the shapeOverride. This will return the shape type override for this dependency or -1
        /// if the shape was not overridden (and the 'common' shape should be used).
        /// </summary>
        public int ShapeOverride
        {
            get
            {
                return this.shapeOverride;
            }
        }

        /// <summary>
        /// This is the property for the dependants variable. This property defines ONLY the getter (no setter).
        /// This property getter will return a ReadOnly version of the dependants.Values() collection.
        /// </summary>
        public IList Dependants
        {
            get
            {
                return ArrayList.ReadOnly(new ArrayList(this.dependants.Values));
            }
        }

        /// <summary>
        /// This constructs the dependency from the specified title with no dependants. This constructor simply
        /// calls the "this(title, -1)" constructor.
        /// </summary>
        /// <param name="title">The title for the dependency</param>
        /// <exception cref="ArgumentNullException">if title is null</exception>
        /// <exception cref="ArgumentException">if title is empty (trim'd)</exception>
        public Dependency(string title) : this(title, -1)
        {
        }

        /// <summary>
        /// This constructs the Dependency from the specified title and allows for a shape override. This constructor
        /// simply saves the parameters to the like named variables.
        /// </summary>
        /// <param name="title">The title for the dependency</param>
        /// <param name="shapeOverride">A shape type that will be used to override the normal one</param>
        /// <exception cref="ArgumentNullException">if either title is null</exception>
        /// <exception cref="ArgumentException">if title is empty (trim'd)</exception>
        /// <exception cref="ArgumentOutOfRangeException">if shapeOverride is 0 or &lt; -1 (ie it must be
        /// positive or -1)</exception>
        public Dependency(string title, int shapeOverride)
        {
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            if (title.Trim() == string.Empty)
            {
                throw new ArgumentException("title should not be empty.");
            }
            if (shapeOverride < -1 || shapeOverride == 0)
            {
                throw new ArgumentOutOfRangeException("shapeOverride should be either -1 or positive.");
            }
            this.title = title;
            this.shapeOverride = shapeOverride;
        }

        /// <summary>
        /// This method is the indexer for the dependants variable and defines both setter and getter actions.
        /// The getter should return null if the title was not found. The setter should create the title/dependency
        /// relation if it doesn't currently exist or replaces the existing dependency if the title is found.
        /// Note: this fully ignores the title in the Dependency.
        /// </summary>
        /// <param name="title">The title of the dependency that this dependency is dependant upon.</param>
        /// <exception cref="ArgumentNullException">if either title is null</exception>
        /// <exception cref="ArgumentException">if title is empty (trim'd)</exception>
        /// <exception cref="ArgumentNullException">if value is null</exception>
        public Dependency this[string title]
        {
            get
            {
                if (title == null)
                {
                    throw new ArgumentNullException("title");
                }
                if (title.Trim() == string.Empty)
                {
                    throw new ArgumentException("title should not be empty.");
                }
                return this.dependants[title] as Dependency;
            }
            set
            {
                if (title == null)
                {
                    throw new ArgumentNullException("title");
                }
                if (title.Trim() == string.Empty)
                {
                    throw new ArgumentException("title should not be empty.");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("dependant");
                }
                this.dependants[title] = value;
            }
        }

        /// <summary>
        /// This method allows the application to add a new dependency to this dependant. This method will take
        /// the dependant's title and store it as the key with the dependant itself as the value.
        /// </summary>
        /// <exception cref="ArgumentNullException">if the dependant is null</exception>
        /// <exception cref="ArgumentException">if the title already exists</exception>
        public void AddDependant(Dependency dependant)
        {
            if (dependant == null)
            {
                throw new ArgumentNullException("dependant");
            }
            if (this.dependants.Contains(dependant.Title))
            {
                throw new ArgumentException("dependant already exists.");
            }
            this.dependants.Add(dependant.Title, dependant);
        }

        /// <summary>
        /// This method will remove a Dependency (as identified by it's title) from the list. This method should
        /// remove from the map the dependency that is identified by the title. Please note that no attempt should
        /// be made to verify that the dependency's title does in fact match the one removed.
        /// </summary>
        /// <param name="title">The dependency's title to remove</param>
        /// <exception cref="ArgumentNullException">if dependant is null or the dependant's title is null</exception>
        /// <exception cref="ArgumentException">if the title (trim'd) is empty</exception>
        public void RemoveDependant(string title)
        {
            if (title == null)
            {
                throw new ArgumentNullException("title");
            }
            if (title.Trim() == string.Empty)
            {
                throw new ArgumentException("title should not be empty.");
            }
            this.dependants.Remove(title);
        }

    }

}
